/*
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.rt;

import holoedit.Main;
import holoedit.data.HoloPoint;
import holoedit.data.HoloSpeaker;
import holoedit.data.HoloTrack;
import holoedit.data.HoloTraj;
import holoedit.data.HoloWaveForm;
import holoedit.data.WaveFormInstance;
import holoedit.util.Ut;
import java.awt.Color;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;
import java.util.EventObject;
import java.util.Vector;

import javax.swing.JComboBox;

import com.apple.dnssd.DNSSD;
import com.apple.dnssd.DNSSDException;
import com.illposed.osc.OSCListener;
import com.illposed.osc.OSCMessage;
import com.illposed.osc.OSCPortIn;
import com.illposed.osc.OSCPortOut;

public abstract class Connection implements Runnable, OSCListener, ConnectionListener {
	public String PROTOCOL_VERSION = "0.1";
	protected static final boolean VERBOSE_IN = false;
	protected static final boolean VERBOSE_OUT = false;
	protected static final int ORDER_NONE = -1;
	protected static final int ORDER_STOP = 0;
	protected static final int ORDER_PLAY = 1;
	protected static final int ORDER_PAUSE = 2;
	protected static final int ORDER_RESUME = 3;
	protected static final int ORDER_RECORD = 4;
	protected static final int CUE_OFFSET = 150; // limit number of tracks to 24 ( because of SPECIAL CUES management )
	protected boolean open = false;
	protected boolean autostop = false;
	protected boolean playing = false;
	protected boolean recording = false;
	protected boolean paused = false;
	protected boolean loop = false;
	protected int out = 13008;
	protected int in = 13005;
	protected String address = "localhost";
	protected OSCPortOut sender;
	protected OSCPortIn receiver;
	protected String keyOut = "/holospat";
	protected String keyIn = "/holoedit";
	protected Thread runner;
	protected int counter;
	protected int end;
	protected int beg;
	protected int total;
	protected boolean firstCue;
	protected Main mainRef;
	protected boolean looping = false;
	protected boolean preloading;
	protected long oldDate = -1, date = -1;
	protected int oversleep = 0;
	protected String currentEditName = "holo.edit";
	protected String currentSpatDomain = "";
	protected String currentSpatName = "";
	protected Vector<HoloTraj> recTrajs = new Vector<HoloTraj>();
	protected int recBegDate = 0;
	protected int loopNum = 0;
	protected int order;
	protected HoloTrack currentTrack;
	protected ArrayList<Object> templist = new ArrayList<Object>();

	public Connection(Main m)
	{
		mainRef = m;
	}
		
	public abstract void open();

	public abstract void close();

	protected abstract void newSender();
	
	protected abstract void newReceiver();	
	
	protected void treatPacket(Date msgDate, String key, Object[] msg)
	{ 
		Vector<String> addr= new Vector<String>(0,1);
		String node;
		String[] keys =  key.split("/");
		
		for(String k : keys)
				addr.add(k);
					
		if(VERBOSE_IN)
		{
			System.out.print("OSCIN : "+key);
			//for(String k:keys)
			//	System.out.print("-"+k+"-");
			//System.out.println();
			for(Object a:msg)
				System.out.print(" "+a.toString());
			System.out.println();
		}
		
		
		if(addr.size()==0)
		{
			System.out.println("Holo-Edit : Unrecognized OSC Message check address ...");
			return;
		}
		
		//remove the first empty element in Vector
		node = addr.remove(0);
		
		node = addr.remove(0);
		// special to the first key to be compared with main HoloEdit Key
		node = "/"+ node;

		if(node.equalsIgnoreCase(keyIn))
		{
			
			if(addr.size()==0)
				return;
			node = addr.remove(0);
			
			// ---- recorder ----
			if (node.equalsIgnoreCase("recorder"))
			{
				if(addr.size()==0)
					return;
				node = addr.remove(0);
				
				int tkNum = Integer.parseInt(node);
				boolean found = false;
				
				if(addr.size()==0)
					return;
				node = addr.remove(0);
				
				if (node.equalsIgnoreCase("in"))
					treatRecord(tkNum,msg);
				if (node.equalsIgnoreCase("segment"))
					treatRecordSegment(tkNum);
				else if (node.equalsIgnoreCase("enable"))
				{
					boolean tkRecOn = (Integer)msg[0] > 0;
					for(int p = 0 ; p < mainRef.gestionPistes.getNbTracks() && !found ; p++)
					{
						HoloTrack tk = mainRef.gestionPistes.getTrack(p);
						if(tk.getNumber()==tkNum)
						{
							found = true;
							tk.setRecEnable(tkRecOn);
							if(tkRecOn)
								tk.setVisible(true);
						}
					}
				}
			}
			// ---- transport ----
			else if(node.equalsIgnoreCase("transport"))
			{
				if(addr.size()==0)
					return;
				node = addr.remove(0);
				
				if (node.equalsIgnoreCase("play"))
				{	
					order = ORDER_PLAY;
					play();
				}
				else if (node.equalsIgnoreCase("resume"))
				{
					order = ORDER_RESUME;
					paused = true;
					pause();
				}
				else if (node.equalsIgnoreCase("pause"))
				{
					order = ORDER_PAUSE;
					paused = false;
					pause();
				}
				else if (node.equalsIgnoreCase("stop"))
				{
					order = ORDER_STOP;
					stop();
				}
				else if (node.equalsIgnoreCase("record") && msg.length > 0)
				{
					order = ORDER_RECORD;
					record((Integer)msg[0] > 0);
				}
				
				else if (node.equalsIgnoreCase("position"))
					position();
				else if (node.equalsIgnoreCase("update"))
					spatUpdate();
				else if (node.equalsIgnoreCase("time") && msg.length > 0)
					setTime((Integer)msg[0]);
				else if (node.equalsIgnoreCase("begtime") && msg.length > 0)
					setBegTime((Integer)msg[0]);
				else if (node.equalsIgnoreCase("endtime") && msg.length > 0)
					setEndTime((Integer)msg[0]);
				else if (node.equalsIgnoreCase("totaltime") && msg.length > 0)
					setTotalTime((Integer)msg[0]);
				else if (node.equalsIgnoreCase("loop") && msg.length > 0)
					setLoop((Integer)msg[0]);
			}
			
			// ---- connection ----
			else if(node.equalsIgnoreCase("connection"))
			{
				
				if(addr.size()==0)
					return;
				
				node = addr.remove(0);
				
				
				if (node.equalsIgnoreCase("key") && msg.length > 0)
				{	
						keyOut = (String)msg[0];
				} 
				else if (node.equalsIgnoreCase("querykey"))
					send(keyOut+"/connection/holoedit/key",new Object[]{keyIn});
				else if (node.equalsIgnoreCase("connect"))
				{	
					boolean connect = true;
					if(msg.length > 0)
						connect = (Integer)msg[0] > 0;
					if(connect)
						open();
					else
						close();
				} 
			}
			
			
			// ---- soundpool ----
			else if(node.equalsIgnoreCase("soundpool"))
			{
				if(addr.size()==0)
					return;
				node = addr.remove(0);
				
				if (key.equalsIgnoreCase("preload"))
					preload();
			}
			
			// ---- tracks ----
			//TODO : wildcard support pour OSCin
			else if(node.equalsIgnoreCase("track"))
			{
				if(addr.size()==0)
					return;
				node = addr.remove(0);
				
				int tkNum = Integer.parseInt(node);
				boolean found = false;
				
				if(addr.size()==0)
					return;
				node = addr.remove(0);
				
				if((node.equalsIgnoreCase("visible")||node.equalsIgnoreCase("on")) && msg.length > 0)
				{
					for(int p = 0 ; p < mainRef.gestionPistes.getNbTracks() && !found ; p++)
						if(mainRef.gestionPistes.getTrack(p).getNumber()==tkNum)
						{
							found = true;
							mainRef.gestionPistes.changeVisible(p);
							mainRef.gestionPistes.ts.checkVisible[p].check(((Integer)msg[0])>0);
						}
					mainRef.room.display();
				}
			}
			
			
			else if (key.equalsIgnoreCase(keyIn+"/speaker/queryspeakers"))
				sendSpeakers();
		}
			else 
				System.out.println("Holo-Edit doesn't understand "+key + " " + msg);
		
	}
	
	protected void treatRecord(int tkNum, Object[] msg)
	{
		int loopDur = end - beg;
		int date = counter;
		boolean editable=false;
		if(msg.length < 3 || !recording)
			return;
		float x = (Float)msg[0];
		float y = (Float)msg[1];
		float z = (Float)msg[2];
		if(msg.length > 3)
			editable = (Integer)msg[3] > 0;
		if(msg.length > 4)
			date = (Integer)msg[4];
		
			
		boolean found = false, empty = false;
		HoloTraj ht;
		for(int p = 0 ; p < mainRef.gestionPistes.getNbTracks() && !found ; p++)
		{
			HoloTrack tk = mainRef.gestionPistes.getTrack(p);
			if(tk.getNumber()==tkNum && tk.isRecEnable())
			{
				found = true;
				ht = recTrajs.get(p);
				empty = ht.isEmpty();
				if(empty)
				{
					ht.add(new HoloPoint(x,y,z,date,true));
					tk.addTraj(ht);
					tk.update();
				}else
					ht.add(new HoloPoint(x,y,z,date,editable));
				
			}
		}
		
	}
	
	
	protected void treatRecordSegment(int tkNum)
	{
		int loopDur = end - beg;
		int date = counter;
		
		boolean found = false;
		for(int p = 0 ; p < mainRef.gestionPistes.getNbTracks() && !found ; p++)
		{
			HoloTrack tk = mainRef.gestionPistes.getTrack(p);
			if(tk.getNumber()==tkNum && tk.isRecEnable())
			{
				HoloTraj ht = new HoloTraj();
				if(!recTrajs.get(p).isEmpty())
					recTrajs.get(p).lastElement().setEditable(true);
				recTrajs.set(p, ht);
			}
		}
		
	}
	
	protected void send(String key, String msg) {
		if (open)
			try
			{
				if(sender != null)
					sender.send(new OSCMessage(key,new Object[]{msg}));
				if(VERBOSE_OUT)
					System.out.println("OSCOUT : "+key+" "+msg);
			} catch (IOException e) {}
	}
	
	public void send(String key, Object[] msg) {
		if (open)
			try
			{
				if(sender != null)
					sender.send(new OSCMessage(key,msg));
				if(VERBOSE_OUT){
					System.out.print("OSCOUT : "+key);
					for(Object a:msg)
						System.out.print(" "+a.toString());
					System.out.println();
				}
				mainRef.transport.setConnectionStatus("@"+address+":"+out, true);
			} catch (IOException e) {
				System.err.println("send io exception");
				mainRef.transport.setConnectionStatus(" -- ", false);
			}
	}
	
	public void send(String msg) {
		if (open)
			send(keyOut,msg);
	}
	
	public void acceptMessage(Date msgDate, OSCMessage msg) {
		if (open)
			treatPacket(msgDate,msg.getAddress(),msg.getArguments());	
	}

	public int getIn() {
		return in;
	}

	public void setIn(int in) {
		if(this.in != in)
		{
			this.in = in;
			newReceiver();
		}
	}

	public int getOut()	{
		return out;
	}

	public void setOut(int out)
	{
		if(this.out != out)
		{
			this.out = out;
			newSender();
		}
	}
	
	public void setAddress(String a)
	{
		if(!this.address.equalsIgnoreCase(a))
		{
			try
			{
				InetAddress.getByName(address);
				address = a;
			} catch (UnknownHostException e)
			{
				address = "localhost";
				mainRef.transport.setConnectionStatus(" -- ", false);
				System.err.println(e.getMessage());
			}
			newSender();
		}
	}

	public void setKeyIn(String k)
	{
		if(!this.keyIn.equalsIgnoreCase(k))
		{
			keyIn = k;
			newReceiver();
		}
	}
	
	public String getKeyIn()
	{
		return keyIn;
	}
	
	public void setKeyOut(String k)
	{
		if(!this.keyOut.equalsIgnoreCase(k))
		{
			keyOut = k;
			newSender();
		}
	}
	
	public String getKeyOut()
	{
		return keyOut;
	}
	
	public void setTime(int msg)
	{
		try {
			int time = msg;
			if(time >= 0)
			{
				boolean p = playing;
				if(p)
					stop();
				mainRef.score.cursorTime = (int)((float)time / 10);
				mainRef.score.display();
				if(p)
					play();
			}
		} catch (NumberFormatException e) {}
	}
	
	public void setTotalTime(int msg)
	{
		try {
			int time = msg;
			if(time >= 0)
			{
				total = Ut.max(time,mainRef.counterPanel.getDate(3));
				mainRef.counterPanel.setCompteur2(3,total);
				mainRef.score.display();
			}
		} catch (NumberFormatException e) {}
	}
	
	public int getTotalTime()
	{
		return total;
	}
	
	public void setBegin(int b)
	{
		beg = b;
	}
	
	public void setBegTime(int msg)
	{
		try {
			int time = msg;
			if(time >= 0)
			{
				boolean p = playing;
				if(p)
					stop();
				mainRef.counterPanel.setCompteur2(1,time);
				mainRef.room.display();
				if(p)
					play();
			}
		} catch (NumberFormatException e) {
		}
	}
	
	public void setEnd(int e)
	{
		end = e;
	}
	
	public void setEndTime(int msg)
	{
		try {
			int time = msg;
			if(time >= 0)
			{
				boolean p = playing;
				if(p)
					stop();
				mainRef.counterPanel.setCompteur2(2,time);
				mainRef.room.display();
				if(p)
					play();
			}
		} catch (NumberFormatException e) {}
	}
	
	public int getCurrentTime()
	{
		return counter;
	}
	
	public void setCurrentTime(int c)
	{
		counter = c;
	}
	
	public void sendBegin()
	{
		if(open)
			send(keyOut+"/transport/begin",new Object[]{mainRef.counterPanel.getDate(1)});
	}
	
	public void sendEnd()
	{
		if(open)
			send(keyOut+"/transport/end",new Object[]{mainRef.counterPanel.getDate(2)});
	}
	
	public void sendTotal()
	{
		if(open)
			send(keyOut+"/transport/total",new Object[]{mainRef.counterPanel.getDate(3)});
	}
	
	public void setLoop(int msg)
	{
		try {
			int l = msg;
			loop(l > 0);
		} catch (NumberFormatException e) {}
	}
	
	public boolean isOpen()
	{
		return open;
	}
	
	public boolean isPlaying()
	{
		return playing;
	}
	
	public boolean isRecording()
	{
		return recording;
	}
	
	public boolean getAutostop()
	{
		return autostop;
	}
	
	public void setAutostop(boolean a)
	{
		autostop = a;
	}
	
	public void record(boolean b)
	{
		if(open)
		{
			if(!b)
			{
				// stop recording
				for(int i = 0 ; i < recTrajs.size() ; i++)
				{
					HoloTraj ht = recTrajs.get(i);
					ht.remove(0);
				}
				for(HoloTrack tk:mainRef.gestionPistes.tracks)
					tk.update();
				recording = false;
				mainRef.transport.rec.setIcon(mainRef.transport.recOffState);
				if(order != ORDER_RECORD)
					send(keyOut+"/transport/record",new Object[]{0});
				else order = ORDER_NONE;
			} else {
				// start recording
				mainRef.gestionPistes.StoreToUndoAll();
				mainRef.modify();
				recTrajs = new Vector<HoloTraj>();
				if(playing)
				{	
					recBegDate = counter;
				} else {
					recBegDate = mainRef.score.cursorTime;
				}	
				for(int i = 0 ; i < mainRef.gestionPistes.getNbTracks() ; i++)
				{
					HoloTraj ht = new HoloTraj();
					currentTrack = mainRef.gestionPistes.getTrack(i);
					//ht.add(new HoloPoint(0,0,0,recBegDate,true));
					recTrajs.add(ht);
					/*if(currentTrack.isVisible() && !currentTrack.isLocked() && currentTrack.isRecEnable())
					{
						currentTrack.addTraj(ht);
						currentTrack.update();
					}*/
				}
				recording = true;
				mainRef.transport.rec.setIcon(mainRef.transport.recOnState);
				if(order != ORDER_RECORD)
					send(keyOut+"/transport/record",new Object[]{1});
				else order = ORDER_NONE;
			}
		}
	}
	
	public void play()
	{
		//if(open && !playing)
		if(!playing)
		{
			runner = new Thread(this);
			runner.setName("RT-player");
			runner.setPriority(Thread.MAX_PRIORITY);
			// PRELOAD
			preload();
			
			try
			{
				Thread.sleep(500);
			}
			catch (InterruptedException e)
			{
			}
			position();
			sendTotal();
			for (HoloTrack tk : mainRef.gestionPistes.tracks)
			{
				if (tk.waves.isEmpty())
					total = Ut.max(total, tk.getLastDate());
				else
					total = Ut.max(total, Ut.max(tk.getLastDate(), tk.waves.lastElement().getLastDate()));
			}
			if(order != ORDER_PLAY)
				send(keyOut+"/transport/play",new Object[]{"bang"});
			else 
				order = ORDER_NONE;
			autostop = false;
			playing = true;
			firstCue = true;
			loopNum = 0;
			counter = mainRef.score.cursorTime;
			looping = !firstCue;
			oversleep = 0;
			runner.start();
			if (!mainRef.rtDisplay.isAnimating())
				mainRef.rtDisplay.start();

		}
	}
	
	public void stop()
	{
		for(HoloTrack tk:mainRef.gestionPistes.tracks)
			tk.stop(true);
//		if(open && (playing || paused || recording))
		if(playing || paused || recording)
		{
			if(runner != null)
				runner.interrupt();
			if(recording)
				record(false);
			if(order != ORDER_STOP)
				send(keyOut+"/transport/stop",new Object[]{"bang"});
			else order = ORDER_NONE;
			recording = false;
			playing = false;
			paused = false;
			if(mainRef.rtDisplay.isAnimating())
				mainRef.rtDisplay.stop();
			mainRef.transport.start.setIcon(mainRef.transport.stopState);
			mainRef.transport.resume.setIcon(mainRef.transport.resumeState);
			
			send(keyOut+"/track/*/cue",new Object[]{0});
		}
		mainRef.room.display();
	}
	
	public void stopAndPlay()
	{
		// STOP
		runner.interrupt();
		send(keyOut+"/transport/jump",new Object[]{"bang"});
		send(keyOut+"/track/*/cue",new Object[]{0});
		runner = new Thread(this);
		runner.setName("RT-player");
		runner.setPriority(Thread.MAX_PRIORITY);
		// PRELOAD
		preload();
		try
		{
			Thread.sleep(500);
		}
		catch (InterruptedException e) {}
		mainRef.transport.start.setIcon(mainRef.transport.playState);
		// PLAY
		for (HoloTrack tk : mainRef.gestionPistes.tracks)
		{
			if (tk.waves.isEmpty())
				total = Ut.max(total, tk.getLastDate());
			else
				total = Ut.max(total, Ut.max(tk.getLastDate(), tk.waves.lastElement().getLastDate()));
		}
		firstCue = true;
		counter = mainRef.score.cursorTime;
		looping = !firstCue;
		loopNum = 0;
		oversleep = 0;
		runner.start();
	}
	
	public void pause()
	{
	//	if(open)
	//	{
			if(paused)
			{
				if(order != ORDER_RESUME)
					send(keyOut+"/transport/resume",new Object[]{"bang"});
				else order = ORDER_NONE;
				mainRef.transport.resume.setIcon(mainRef.transport.resumeState);
				paused = false;
			} else {
				if(order != ORDER_PAUSE)
					send(keyOut+"/transport/pause",new Object[]{"bang"});
				else order = ORDER_NONE;
				mainRef.transport.resume.setIcon(mainRef.transport.pauseState);
				paused = true;
			}
	//	}
	}
	
	public void preload()
	{
		int tkNum;
		int tkNb = mainRef.gestionPistes.getNbTracks();
		preloading = true;
		beg = mainRef.counterPanel.getDate(1);
		end = mainRef.counterPanel.getDate(2);
		if(open)
		{
			try{
				sendPaths();
				sendVisible();
				 
					Thread.sleep(10);
				
				for(HoloTrack tk:mainRef.gestionPistes.tracks)
				{
					tkNum = tk.getNumber();
					for(int i = 1 ; i <= tk.waves.size() ; i++)
						//tk.waves.get(i-1).preload(tk,100+(i*tkNb)+tkNum,mainRef.score.cursorTime, beg,end);
						preloadWaveForm(tk.waves.get(i-1),tk,CUE_OFFSET+(i*tkNb)+tkNum,mainRef.score.cursorTime, beg,end);
				}
			} catch (InterruptedException e) {
			// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		mainRef.gestionPistes.preparePlay();
		preloading = false;
	}
	
	public void preloadFirst()
	{
		int tkNum;
		int tkNb = mainRef.gestionPistes.getNbTracks();
		preloading = true;
		beg = mainRef.counterPanel.getDate(1);
		end = mainRef.counterPanel.getDate(2);
		if(open)
		{
			try{
				sendPaths();
				sendVisible();
				 
					Thread.sleep(100);
				
				for(HoloTrack tk:mainRef.gestionPistes.tracks)
				{
					tkNum = tk.getNumber();
					for(int i = 1 ; i <= tk.waves.size() ; i++)
						preloadWaveForm(tk.waves.get(i-1),tk,CUE_OFFSET+(i*tkNb)+tkNum,mainRef.score.cursorTime, beg,end);
				}
			} catch (InterruptedException e) {
			// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		mainRef.gestionPistes.preparePlay();
		preloading = false;
	}
	
	public void preloadWaveForm(WaveFormInstance w,HoloTrack tk, int cueNb, int cursorTime, int begL, int endL)
	{
		HoloWaveForm hwf = w.getWave();
		if(hwf == null)
			w.update();
		if(hwf == null)
			return;
		w.loopDefined = -1;
		w.cue = cueNb;
		w.begLoop = begL;
		w.endLoop = endL;
		//String keyOut = gpRef.mainRef.connection.getKeyOut();
		int tkNb = tk.getNumber();
		int cueOffset = w.SPEC_CUE_OFFSET + w.SPEC_CUE_NB * tkNb;
		if(Ut.between(cursorTime,w.getFirstDate(),w.getLastDate()))
		{
			if(w.getLastDate() > w.endLoop && cursorTime < w.endLoop)
			{
				send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueOffset+w.CURSOR_CUE_LOOP,hwf.getStripPathWoQuote(),(cursorTime-w.getFirstDate())*10,(w.endLoop-w.getFirstDate())*10});
				w.loopDefined = w.LOOP_END_CUE;
			}
				send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueOffset+w.CURSOR_CUE,hwf.getStripPathWoQuote(),(cursorTime-w.getFirstDate())*10});
		}
		if(Ut.between(w.getFirstDate(),w.begLoop,w.endLoop) && !Ut.between(w.getLastDate(),w.begLoop,w.endLoop))
		{
			send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueOffset+w.LOOP_END_CUE,hwf.getStripPathWoQuote(),0,(w.endLoop-w.getFirstDate())*10});
			send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueOffset+w.LOOP_AFTER_CUE,hwf.getStripPathWoQuote(),(w.endLoop-w.getFirstDate())*10});
			w.loopDefined = w.LOOP_END_CUE;
		} else if(!Ut.between(w.getFirstDate(),w.begLoop,w.endLoop) && Ut.between(w.getLastDate(),w.begLoop,w.endLoop))
		{
			send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueOffset+w.LOOP_BEGIN_CUE,hwf.getStripPathWoQuote(),(w.begLoop-w.getFirstDate())*10});
			w.loopDefined = w.LOOP_BEGIN_CUE;
		} else if(w.getFirstDate() < w.begLoop && w.getLastDate() > w.endLoop)
		{
			send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueOffset+w.LOOP_IN_CUE,hwf.getStripPathWoQuote(),(w.begLoop-w.getFirstDate())*10,(w.endLoop-w.getFirstDate())*10});
			send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueOffset+w.LOOP_AFTER_CUE,hwf.getStripPathWoQuote(),(w.endLoop-w.getFirstDate())*10});
			w.loopDefined = w.LOOP_IN_CUE;
		}
		
			send(keyOut+"/track/"+tkNb+"/preload",new Object[]{cueNb,hwf.getStripPathWoQuote()});
		return;
	}
	
	public void playWaveForm(WaveFormInstance w, HoloTrack tk, int time, boolean firstCue, boolean loop, boolean looping)
	{
		if(Ut.between(time,w.getFirstDate(),w.getLastDate()))
		{
			int tkNb = tk.getNumber();
			int cueOffset = w.SPEC_CUE_OFFSET + w.SPEC_CUE_NB * tkNb;
			if(firstCue)
			{
				if(loop && w.loopDefined == w.LOOP_END_CUE)
				{
					send(keyOut+"/track/"+tkNb+"/cue",new Object[]{cueOffset+w.CURSOR_CUE_LOOP});
					return;
				}
				send(keyOut+"/track/"+tkNb+"/cue", new Object[]{cueOffset+w.CURSOR_CUE});
				return;
			} else if(loop) {
				if(time == w.begLoop && looping) {
					if(w.loopDefined == w.LOOP_BEGIN_CUE)
					{
						send(keyOut+"/track/"+tkNb+"/cue", new Object[]{cueOffset+w.LOOP_BEGIN_CUE});
						return;
					}
					else if(w.loopDefined == w.LOOP_IN_CUE)
					{
						send(keyOut+"/track/"+tkNb+"/cue", new Object[]{cueOffset+w.LOOP_IN_CUE});
						return;
					}
					else if(time == w.begTime)
					{
						send(keyOut+"/track/"+tkNb+"/cue", new Object[]{w.cue});
						return;
					}
				} else if(time == w.begTime) {
					if(w.loopDefined == w.LOOP_END_CUE)
					{
						send(keyOut+"/track/"+tkNb+"/cue", new Object[]{cueOffset+w.LOOP_END_CUE});
						return;
					}
					send(keyOut+"/track/"+tkNb+"/cue", new Object[]{w.cue});
					return;
				}
			} else if(time == w.begTime) {
				w.loopDefined = -1;
				send(keyOut+"/track/"+tkNb+"/cue", new Object[]{w.cue});
				return;
			} else if(time == w.endLoop && (w.loopDefined == w.LOOP_END_CUE || w.loopDefined == w.LOOP_IN_CUE))
			{
				send(keyOut+"/track/"+tkNb+"/cue", new Object[]{cueOffset+w.LOOP_AFTER_CUE});
				return;
			}
		}
		return;
	}
	
	public void spatUpdate()
	{
		//String tmp = "tks "+mainRef.gestionPistes.getNbTracks();
		templist.clear();
		
		send(keyOut+"/config/tracknum",new Object[]{mainRef.gestionPistes.getNbTracks()});
		
		for(HoloTrack tk:mainRef.gestionPistes.tracks)
			templist.add(tk.getNumber());
		send(keyOut+"/config/tracklist",templist.toArray());
			//tmp += " " + tk.getNumber();

		for(HoloTrack tk:mainRef.gestionPistes.tracks)
		{
			Color c = tk.getColor();
			String na = tk.getName();
			/*if(na.indexOf(' ') != -1)
				na = "\""+na+"\"";*/
			send(keyOut+"/track/"+tk.getNumber()+"/name",new Object[]{na});
			send(keyOut+"/track/"+tk.getNumber()+"/color",new Object[]{c.getRed(),c.getGreen(),c.getBlue()});
			//send("tkup "+tk.getNumber()+" "+na+" "+c.getRed()+" "+c.getGreen()+" "+c.getBlue());
		}
		position();
		preloadFirst();
	}
	
	public void loop(boolean l)
	{
		loop = l;
//		if(open)
	//	{
			send(keyOut+"/transport/loop",new Object[]{(loop ? 1 : 0)});
			if (loop)
				mainRef.transport.loop.setIcon(mainRef.transport.loopOnState);
			else
				mainRef.transport.loop.setIcon(mainRef.transport.loopOffState);

	//	}
	}
	
	public boolean getLoop()
	{
		return loop;
	}
	
	public boolean changeLoop()
	{
		loop(!loop);
		return getLoop();
	}

	public void playstop()
	{
	//	if(open)
	//	{
			if(playing || autostop) stop();
			else play();
	//	}
	}

	public void position()
	{
		HoloPoint p;
		counter = mainRef.score.cursorTime;
		send(keyOut+"/transport/time",new Object[]{counter});
		for(int i = 0, last = mainRef.gestionPistes.getNbTracks() ; i < last ; i++)
		{
			currentTrack = mainRef.gestionPistes.getTrack(i);
			currentTrack.stop(true);
			if(currentTrack.isVisible())
			{
				if((p = currentTrack.getPointPlay(counter)) != null)
					send(keyOut+"/track/"+currentTrack.getNumber()+"/xyz",new Object[]{p.x,p.y,p.z});
				/*String vc = currentTrack.getLastValueAt(counter);
				if(!vc.equalsIgnoreCase("none"))
					send(keyOut+"/track/"+currentTrack.getNumber()+"/xyz",new Object[]{vc});*/
			}
		}
		mainRef.room.display();
	}
	
	public void sendVisible()
	{
		for(HoloTrack t:mainRef.gestionPistes.tracks)
		{
			send(keyOut+"/track/"+t.getNumber()+"/visible",new Object[]{(t.isVisible() ? 1 : 0)});
			send(keyOut+"/track/"+t.getNumber()+"/active",new Object[]{(t.isVisible() ? 1 : 0)});
		}
		
	}

	public void sendPaths()
	{
		Vector<String> paths = new Vector<String>();
		for(HoloWaveForm hwf:mainRef.gestionPistes.soundPool.getSounds())
		{	
			String p = hwf.getPathWoQuote();
			if(!paths.contains(p))
				paths.add(p);
		}
		send(keyOut+"/soundpool/paths",new Object[]{paths.size()});
		for(int i = 0 ; i < paths.size() ; i++)
			send(keyOut+"/soundpool/path",new Object[]{i,paths.get(i)});
	}
	
	public String getAddress()
	{
		return address;
	}
	
	public void run()
	{
		// si modif, modifier galement HoloPlayer
		try
		{
			while(playing)
			{
				HoloPoint p;
				while(preloading)
				{
					System.out.println("preloading...");
					Thread.sleep(10);
				}
				if(!paused)
				{
					oldDate = new Date().getTime();
					if(loop && counter == end+1)
					{
						send(keyOut+"/transport/looping",new Object[]{"bang"});
						counter = beg;
						looping = true;
						loopNum++;
					} else if(counter > total && !recording){
						send(keyOut+"/transport/stop",new Object[]{"bang"});
						playing = false;
						paused = false;
						autostop = true;
					}
						
					send(keyOut+"/transport/time",new Object[]{counter});
					
					for(int i = 0, last = mainRef.gestionPistes.getNbTracks() ; i < last ; i++)
					{
						currentTrack = mainRef.gestionPistes.getTrack(i);
						if(currentTrack.isVisible())
						{
							if((recording && !currentTrack.isRecEnable()) || !recording)
							{
								if((p = currentTrack.getPointPlay(counter)) != null)
									send(keyOut+"/track/"+currentTrack.getNumber()+"/xyz",new Object[]{p.x,p.y,p.z});
								/*String vc = currentTrack.getValueAt(counter,autostop,looping);
								if(!vc.equalsIgnoreCase("none"))
									send(keyOut+"/track/"+currentTrack.getNumber()+"/xyz",new Object[]{vc});*/
							}
							for(WaveFormInstance w : currentTrack.waves)
							{	
								playWaveForm(w,currentTrack, counter, firstCue, loop, looping);
							}
						}
					}
					firstCue = false;
					looping = false;
					counter ++;
					date = new Date().getTime();
					int delta = (int)(date - oldDate);
					if(delta < 10)
					{
						delta = 10 - delta;
						if(oversleep > delta)
						{
							oversleep -= delta;
						} else {
							delta -= oversleep;
							oversleep = 0;
							date = new Date().getTime();
							Thread.sleep(delta);
							oversleep += (int)(new Date().getTime() - date - delta);
						}
					}
				}else
				{
					Thread.sleep(10);
				}
			}
		} catch (InterruptedException e) {}
		mainRef.transport.start.setIcon(mainRef.transport.stopState);
	}
	
	public String toString()
	{
		return "\t<osc portIn=\"" + mainRef.connection.getIn() + "\"" + " portOut=\"" + mainRef.connection.getOut() + "\""
		+ " address=\"" + mainRef.connection.getAddress() + "\"" + " open=\"" + mainRef.connection.isOpen() + "\""
		+ " keyIn=\"" + keyIn + "\"" + " keyOut=\"" + keyOut + "\"" + "/>\n";		
	}

	public void sendSpeakers()
	{
		HoloSpeaker sp;
		for(int i = 0 ; i < mainRef.gestionPistes.speakers.size() ; i++)
		{
			sp = mainRef.gestionPistes.speakers.get(i);
			send(keyOut+"/speaker/"+(i+1)+"/xyz",new Object[]{sp.X,sp.Y,sp.Z});
		}
	}
	
	public void oscAddressChanged(EventObject event, String newAdress){
		setAddress(newAdress);
	}	
	public void oscPortOutChanged(EventObject event, int newOscPortOut){
		setOut(newOscPortOut);
	}
	public void connectToService(EventObject event, String serviceName){
		if (mainRef.bonjour){
			try {
				BonjourConnection bonjourConnection =  (BonjourConnection) mainRef.connection;
				DNSSD.resolve(0, 0, serviceName, bonjourConnection.registeringService 
						, bonjourConnection.spatmap.get(serviceName) , bonjourConnection);
				
			} catch (DNSSDException e) {
				System.err.println("OSCConnection : DNSSDException while trying to register.");
				e.printStackTrace();
			}
		}
	}	
	public void disconnectCalled(EventObject event){
		close();
	}
}